##################################################################################################
######################################## CLASS DEFINITION ######################################## 
##################################################################################################
MixedAssignment <- setRefClass(
  Class = "MixedAssignment", 
  fields = list(
    variables = "list",
    values = "list"
  )
)

##################################################################################################
###################################### SETTERS AND GETTERS #######################################
##################################################################################################
MixedAssignment$accessors(c("values", "variables"))

##################################################################################################
########################################### METHODS ##############################################
##################################################################################################

##################################################################################################

# Initializes the Assignment.
#
# Args:
#   elem: The first element in the Queue (non mandatory argument)
#
# Returns:
#   The initialized Assignment object.

#MixedAssignment$methods (
#  initialize=function(vars,vals){
#    variables <<- vars
#    values <<- vals
#  }  
#)
  
##################################################################################################

# Gets the size of the assigment (number of variables). 
#
# Args:
#
# Returns:
#   The size of the assignment.

MixedAssignment$methods (
  size = function() {
    return(length(variables))   
  }
)

##################################################################################################

# Shows relevant information about the assignment: #variables, #values
#
# Args:
#
# Returns: 
#   No arguments are returned. Only the information is displayed on the screen. 

MixedAssignment$methods (
  show = function() {
    for (i in 1:size()) {
      if (variables[[i]]$isContinuous()){
        cat("Continuous  ", variables[[i]]$getName(),":", values[[i]], "\n")
      }
      else {
        cat("Categorical ", variables[[i]]$getName(),": ")
        cat(values[[i]],"\n")
      }   
    }
  }
)

##################################################################################################

# Gets the variable at a position 
#
# Args:
#   i: The position of the variable.
#
# Returns:
#   The variable at position i

MixedAssignment$methods (
  getVarAt = function(i) {
    return(variables[[i]])
  }
)

##################################################################################################

# Gets the value at a position  
#
# Args:
#   i: The position of the value
#
# Returns:
#   The value at position i

MixedAssignment$methods (
  getValueAt = function(i) {
    return(values[[i]])
  }
)

##################################################################################################

# Gets the value of a variable in the assignment. 
#
# Args:
#   var: The variable
#
# Returns:
#   The value of the variable in the assignment.

MixedAssignment$methods (
  getValue = function(var) {
    for(i in 1:size()) {   
      #if (var$equals(getVarAt(i))) {
      if (identical(getVarAt(i),var)) {
        return(getValueAt(i))
      }
    }
    return (NULL)   
  }
)


##################################################################################################

# Removes the element (variable, value) stored in  a given position.
#
# Args:
#   i: The position of the element.
#
# Returns:
#   The assignment is modified.

MixedAssignment$methods (
  removeElementAt = function(i) {
    setVariables(getVariables()[-c(i)])
    setValues(getValues()[-c(i)])
  }
)

##################################################################################################

# Checks whether or not the variable is evidenced.  
#
# Args:
#   var: The variable to be checked.
#
# Returns:
#   A boolean value indicating whether or not the variable is evidenced.

MixedAssignment$methods (
  isEvidenced = function(variable) {
    if(size()==0) 
      return(FALSE)
    else
      return(length(getValue(variable))==1)
  }
)

##################################################################################################

# Adds a new pair (variable, value) to the assignment. 
#
# Args:
#   variable: The new variable.
#   value: The value of the variable.
#
# Returns:
#   The assignment with the new element. 

MixedAssignment$methods (
  addElement = function(variable,value) {
    variables[[length(variables)+1]] <<-variable
    values[[length(values)+1]] <<- value
  }
)

##################################################################################################

# Sets a value for a variable. 
#
# Args:
#   variable: The variable.
#   value: The new value.
#
# Returns:
#   The modified assignment.

MixedAssignment$methods (
  setValue = function(variable,value) {
      for(i in 1:size()) {   
        #if (variable$equals(getVarAt(i))) {
        if (identical(getVarAt(i),variable)) {
          values[[i]] <<- value
        }
      }
  }
)

##################################################################################################

# Sets a value for a variable located at a position. 
#
# Args:
#   value: The new value.
#   i: The position of the variable to be modified.
#
# Returns:
#   The modified assignment.

MixedAssignment$methods (
  setValueAt = function(value,i) {
      values[[i]] <<- value
  }
)

##################################################################################################

# Updates the assigment with the information contained in a data frame row. 
#
# Args:
#   variables: The variables to be included in the assignment.
#   row: A row of a data frame with the values for the variables. 
#
# Returns:
#   An assignment with the new information.

MixedAssignment$methods (
  loadFromDataFrameRow = function(variables, row) {
    
    for (i in 1:length(variables)){
      if (variables[[i]]$isCategorical())
        addElement(variables[[i]],as.character(row[,i]))
      else
        addElement(variables[[i]],as.numeric(row[,i]))
    }
  }
)

##################################################################################################
